using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

// Our own library
using XNACS1Lib;

namespace AlphabetHunt
{
    /// <summary>
    /// This is the background side scoller. 
    /// 
    /// To use this scroller, pass in the following information at construction time:
    /// 
    ///     backgroundImage: texture image name
    ///     imageWidth: width (in pixel) of the backgroun image, notice that for good visual
    ///                 results, the background image should have resoultion:
    /// 
    ///                     BackgroundImageWidth x 360 (with slightly lower quality)
    ///                     BackgroundImageWidth x 720 or
    ///                     BackgroundImageWidth x 1080
    /// 
    ///     worldSize: world width and height
    /// 
    /// </summary>
    public class CSideScroll
    {
        
        private const float REF_WIDTH = 720.0f;
        private const float kCyclePeriod = 500.0f; // nominal speed causes repeat of REF_WIDTH updates in 500 cycles
        
        private float BackgroundImageWidth;     // Image height should be 720 or 1080
        private String BackgroundImageName;        
        private float kViewPortion;             // image length to View length ratio
        private float kPeriod;                  // nominal speed causes repeat in 1000 of REF_WIDTH updates

        private float m_Unit;
        private float m_CurrentXPosition;

        private float m_ViewWidth;
        private float m_ViewHeight;
        private float m_HalfViewHeight;
        private float m_ScollLength;
        
        private float m_Speed;

        private XNACS1Rectangle m_FirstPage, m_SecondPage;

        /// <summary>
        /// This is the constructor. Notice we must call the base class's constructor
        /// by passing in the coordinate of the lower-left corner, and the width of the world.
        /// <param name="bgImageName"> image name of the bgscrolling image (without the .jpg extension)</param>
        /// <param name="bgImageWidth"> width (in pixels) of the background scrolling image</param>
        /// <param name="worldSize"> width and height of world dimension</param>
        /// </summary>
        public CSideScroll(String bgImageName, int bgImageWidth, Vector2 worldSize) {
 
            BackgroundImageWidth = bgImageWidth;      
            BackgroundImageName = bgImageName;
            kViewPortion = REF_WIDTH/BackgroundImageWidth;   
            kPeriod = kCyclePeriod/kViewPortion;  

            m_ViewWidth = worldSize.X;
            m_ViewHeight = worldSize.Y;
            m_HalfViewHeight = 0.5f * worldSize.Y;

            m_ScollLength = m_ViewWidth / kViewPortion;

            m_CurrentXPosition = 0.0f;
            m_Unit = m_ScollLength / kPeriod;
            m_Speed = m_Unit;

            m_FirstPage = new XNACS1Rectangle(new Vector2(0.0f, m_HalfViewHeight), m_ScollLength, m_ViewHeight, BackgroundImageName);
            m_SecondPage = new XNACS1Rectangle(new Vector2(0.0f, m_HalfViewHeight), m_ScollLength, m_ViewHeight, BackgroundImageName);
        }
        public void UpdateScroll() {
            m_CurrentXPosition += m_Speed;
            if (m_CurrentXPosition > m_ScollLength)
                m_CurrentXPosition -= m_ScollLength;

            if (m_CurrentXPosition < 0.0f)
                m_CurrentXPosition += m_ScollLength;

            UpdateScrollPages();
        }
        public void SetScrollSpeed(float rate) {
            m_Speed = rate * m_Unit;
        }

        private void UpdateScrollPages() {
            // we want to show X ranges from: 
            //             m_CurrentXPosition to rightPos
            // But, the window is fixed: 
            //             0 to m_WorldSize.X
            // so, we move the background rectangle backwards!
            // 
            float left = -m_CurrentXPosition;
            float right = left + m_ScollLength;
            m_FirstPage.CenterX = 0.5f * (left + right);

            float rightPos = m_CurrentXPosition + m_ViewWidth;
            if (rightPos > m_ScollLength) // wrap around
            {
                // wrapped around second half
                left = right;
                right = left + m_ScollLength;
                m_SecondPage.CenterX = 0.5f * (left + right);
                m_SecondPage.Visible = true;
            }
            else
            {
                m_SecondPage.Visible = false;
            }
        }

        // for any object that are in sync with this scroller
        // Objects should be defined and updated according to simple world coordinate (WC)
        // Objects should:
        //     1. call UpdatePosition to move along with the scroller
        
        public float UpdatePosition(float pos)
        {
            return pos -= m_Speed;
        }
    }

}