// #define STUDENT_VERSION
#define INSTRUCTOR_VERSION

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;

#endregion


// 
//
// ALL Media files must be PART of the solution (to be deployed over to the XBOX)!!


namespace InsectsGarden_Program
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class InsectsGarden : XNACS1Base
    {

        // Label A: Constants
        private const float WorldWidth = 40.0f;         // width of world

        private const int INSECT_TIMER_MAX = 200;       // maximum periods an insect can appear
        private const int INSECT_TIMER_BASE = 100;      // minimum periods an insect should appear
        private const int NUM_FLOWERS = 23;             // number of background flowers

        private const float NET_POSY = 1.8f;            // position of the net relative
        private const float NET_SIZE = 1.5f;            // to the happy face hero

        //
        // Label B-1: Information on the hero (purple happy face) and the net
        private XNACS1Rectangle m_Hero;                 // Hero
        private XNACS1Circle m_Net;                     // The Net


        // Label B-2: Insect position, length of appearance and num touched
        private XNACS1Circle m_Insect;                   // the insect
        private int m_InsectTimer;                       // cycles till generate new insect

        // Num touched
        private int m_NumTouched;                         // count of insects touched by net

        /// <summary>
        /// Always set the World coordinate dimension, and create all the objects
        /// Remember: to create the background objects first (the objects to be drawn on top
        /// should be created last!)
        /// </summary>
        protected override void InitializeWorld()
        {
            // Label A: define coordinate and background color
            World.SetWorldCoordinate(new Vector2(-1.0f, -1.0f), WorldWidth);

            XNACS1Base.World.SetBackgroundColor(Color.LawnGreen);

            // Label B: Flower initialization
            for (int i = 0; i < NUM_FLOWERS; i++)
            {
                float x = RandomFloat(World.WorldMax.X); // Random x/y position inside the window
                float y = 0.5f + RandomFloat(World.WorldMax.Y - 1.5f);
                String tex = "flower" + RandomInt(6);  // Texture image name
                XNACS1Rectangle aFlower = new XNACS1Rectangle(new Vector2(x, y), 1.0f, 1.0f, tex);
            }

            // Label C: The Hero
            Vector2 pos = new Vector2(World.WorldMax.X / 2.0f, World.WorldMax.Y / 2.0f);
            m_Hero = new XNACS1Rectangle(pos, 4.0f, 4.0f, "lnet");

            // Label D: The Net
            pos.Y += NET_POSY;
            m_Net = new XNACS1Circle(pos, NET_SIZE);
            m_Net.CenterColor = Color.White;
            m_Net.OutsideColor = Color.White;

            // Label E: The Insect
            m_Insect = null;
            m_InsectTimer = 0;
            m_NumTouched = 0;
        }

        /// <summary>
        /// Update application state and draw the current state of all game pieces
        /// </summary>
        protected override void UpdateWorld()
        {
            // Update state

            // Allows the default game to exit on Xbox 360 and Windows
            if (XNACS1Base.GamePad.Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // Label A: Polling thumbstick state and update hero position
            Vector2 delta = XNACS1Base.GamePad.ThumbSticks.Left;
            bool once = false;

            if (delta.X > 0.0f)
            {
                m_Hero.Texture = "rnet";
                once = true;
            }
            else if (delta.X < 0.0f)
            {
                m_Hero.Texture = "lnet";
                once = true;
            }

            if (once)
                delta.Normalize();
            m_Hero.Center = m_Hero.Center + (delta * 0.5f);   // Move at constant speed

            World.ClampAtWorldBound(m_Hero);
            m_Net.Center = m_Hero.Center;
            m_Net.CenterY += NET_POSY;

            // Label B: Checking the insect: 1. timer condition 2. touched by net
            if (m_InsectTimer > 0)                                      // says insect is visible
            {
                m_InsectTimer--;                                        // counter-dwon to geneate new insect
                Vector2 buzz = new Vector2();
                buzz.X = RandomFloat(-0.2f, 0.2f);                      // compute a small random buzz velocity
                buzz.Y = RandomFloat(-0.2f, 0.2f);
                // performed automatically because ShouldTravel is set to true. 
                // m_Insect.Center = m_Insect.Center + m_Insect.Velocity;   // insect's flying direction
                m_Insect.Center = m_Insect.Center + buzz;               // small buzz movement

                // make sure insect does not leave the window
                World.ClampAtWorldBound(m_Insect);

                // Label B-1: checking on the insect touched by net
                if (m_Net.Collided(m_Insect))
                {
                    m_InsectTimer = -1;                                 // current insect expires
                    m_NumTouched++;                                     // increase count
                    XNACS1Base.PlayACue( "netted");
                }
            }

            // Label C: create new insect if necessary
            if (m_InsectTimer <= 0)                                     // current insect has expired
            {
                if (null != m_Insect)
                    m_Insect.RemoveFromAutoDrawSet();                   // should not draw this anymore!

                Vector2 pos = new Vector2(RandomFloat(World.WorldMax.X, World.WorldMax.Y));
                m_Insect = new XNACS1Circle(pos, 1.0f, "insect7");
                m_InsectTimer = RandomInt(INSECT_TIMER_BASE, INSECT_TIMER_MAX); // random appearance count
                m_Insect.VelocityX = RandomFloat(-1.0f, 1.0f);          // random flying velocity
                m_Insect.VelocityY = RandomFloat(-1.0f, 1.0f);
                m_Insect.ShouldTravel = true;                           // causes automatic update of position
                m_Insect.Velocity.Normalize();
                m_Insect.Velocity *= 0.05f;
                XNACS1Base.PlayACue( "squirt");
            }

            // Label D: Echo status
            XNACS1Base.EchoToBottomStatus("Number of insects touched:" + m_NumTouched + "  Insect Timer: " + m_InsectTimer);
            XNACS1Base.EchoToTopStatus("Insect Position:" + m_Insect.Center);
        }
    }
}