using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

// Our own library
using XNACS1Lib;

namespace SnakeAndLadder_Program
{
    class GameBoardGUI
    {
        private GameBoard m_GameBoard;
        private XNACS1Circle[] m_GoldBars;
        private XNACS1Circle m_TheKid;

        private const float SIZE = 0.95f;
        private const float LABEL_SIZE = 0.2f;
        private static float GOLD_SIZE = 0.25f;

        private static float ROPE_WIDTH = 0.3f;
        private static float LADDER_WIDTH = 0.3f;
        private static float ANCHOR_SIZE = GOLD_SIZE - 0.15f;

        public GameBoardGUI(int numCells, int maxColumn)
        {
            m_GameBoard = new GameBoard(numCells, maxColumn);

            // Now create the game board 
            for (int i = 0; i < m_GameBoard.getTotalCellsOnBoard(); i++)
            {
                Vector2 pos = GetCellPosition(i);
                XNACS1Rectangle cell = new XNACS1Rectangle(pos, SIZE, SIZE);
                cell.OutsideColor = cell.CenterColor = Color.LightSeaGreen;
                cell.Label = i.ToString();
            }
            m_GoldBars = new XNACS1Circle[numCells];

            for (int i = 0; i < m_GameBoard.getTotalCellsOnBoard(); i++)
            {
                m_GoldBars[i] = null;

                if (null != m_GameBoard.getCell(i))
                {
                    Vector2 pos = GetCellPosition(i);
                    pos.X -= 0.2f;
                    pos.Y -= 0.2f;

                    int skip = m_GameBoard.getCell(i).getCellsToSkip();
                    if (skip != 0)
                        CreateSnakeLadder(i, skip);

                    if (m_GameBoard.getCell(i).getGoldPoints() > 0)
                        m_GoldBars[i] = new XNACS1Circle(pos, GOLD_SIZE, "gold");
                }
            }
            m_TheKid = new XNACS1Circle(GetCellPosition(m_GameBoard.getPlayerCellIndex()), 0.4f, "kid");
        }

        private void PlayCue(int skip)
        {
            if (skip > 0)
                XNACS1Base.PlayACue( "GoUp");
            else
                XNACS1Base.PlayACue( "GoDown");
        }

        private Vector2 GetCellPosition(int position)
        {
            int row = position / m_GameBoard.getMaxColumn();
            int col = position % m_GameBoard.getMaxColumn();
            Vector2 p = new Vector2();
            p.Y = row + 0.5f;
            p.X = col + 0.5f;
            if (0 != (row % 2))
                p.X = m_GameBoard.getMaxColumn() - p.X;    // odd count backwards

            return p;
            // return (new Vector2((position % m_GameGoard.getMaxColumn()) + 0.5f, (position / m_GameGoard.getMaxColumn()) + 0.5f));
        }

        public void ChanceIt(PlayStatus stat)
        {
            int dst = m_GameBoard.createRandomSnakeOrLadder();
            if (dst != GameBoard.INVALID_DESTINATION)
            {
                int src = m_GameBoard.getPlayerCellIndex();
                int skip = m_GameBoard.getCell(src).getCellsToSkip();
                PlayCue(skip);
                CreateSnakeLadder(src, skip);
                GoToCell(dst, stat);
            }
        }

        // return true if update is not done (should cal update again)
        public bool UpdatePlayStatus(PlayStatus stat)
        {
            bool once = false;
            if (stat.ValidStatus(m_TheKid.Center))
            {
                m_TheKid.Center = m_TheKid.Center + stat.Velocity;
                once = true;
            }
            else
            {
                m_TheKid.Center = GetCellPosition(m_GameBoard.getPlayerCellIndex());
                stat.Initialize();
                stat.Status = "Arrive at Cell:" + m_GameBoard.getCellContainingPlayer();

                if (null != m_GameBoard.getCellContainingPlayer())
                {
                    if (m_GameBoard.getCellContainingPlayer().getGoldPoints() > 0)
                    {
                        XNACS1Base.PlayACue( "GotGold");
                        // the player has picked up the gold, so record that the player
                        // now has it...
                        stat.Gold += m_GameBoard.getCellContainingPlayer().getGoldPoints();
                        // ...and that it's no longer on the board
                        m_GameBoard.getCellContainingPlayer().setGoldPoints(0);
                        int index = m_GameBoard.getPlayerCellIndex();
                        m_GoldBars[index].RemoveFromAutoDrawSet();
                        m_GoldBars[index] = null;
                    }
                    int skip = m_GameBoard.getCellContainingPlayer().getCellsToSkip();
                    if (skip != 0 && false)
                    {
                        int destCell = m_GameBoard.getPlayerCellIndex() + skip;
                        GoToCell(destCell, stat);
                        once = true;
                        PlayCue(skip);
                    }
                }
            }
            return once;
        }


        private void CreateSnakeLadder(int from, int skip)
        {
            float width;
            String texture;

            if (skip > 0)
            {
                width = LADDER_WIDTH;
                texture = "ladder";
            }
            else
            {
                width = ROPE_WIDTH;
                texture = "snake";
            }
            Vector2 fromPos = GetCellPosition(from);
            Vector2 toPos = GetCellPosition(from + skip);
            XNACS1Rectangle r = new XNACS1Rectangle(fromPos, toPos, width, texture);
        }

        public void MoveBy(int num, PlayStatus stat)
        {
            int dstCell = (m_GameBoard.getPlayerCellIndex() + num) % m_GameBoard.getTotalCellsOnBoard();
            GoToCell(dstCell, stat);
        }

        private void GoToCell(int c, PlayStatus stat)
        {
            stat.Status += "Moving to: " + c;
            stat.InitStateFor(GetCellPosition(m_GameBoard.getPlayerCellIndex()), GetCellPosition(c));
            m_GameBoard.setPlayerCellByIndex(c);
        }

        /// Simple pass-through to the data object
        public int TotalGoldOnBoard()
        {
            return m_GameBoard.getTotalGoldOnBoard();
        }
    }
}
