using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

// Our own library
using XNACS1Lib;

namespace SnakeAndLadder_Program
{
    class GameBoardGUI
    {
        enum BoardCellContent {
            HasGold,
            HasNothing
        };
        
        // Label A: instance variable describing the game board dimension
        private BoardCellContent[] m_GameBoard;         // Game board is stored as a 1D array
        private XNACS1Circle[] m_GoldBars;              // For showing gold bars
        private int m_MaxColumn;                        // number of column on the board
        private int m_TotalNumCells;                    // Total number game cells


        // Label B: instance variable describing the position of the hero (index and (x,y))
        private int m_CurrentIndex;                     // current index position of the hero
        private XNACS1Circle m_Hero;                    // The kid

        // Label C: constants for drawing the cells
        private const float SIZE = 0.95f;               // Size of each game cell
        private const float LABEL_SIZE = 0.2f;          // size of number label in the cell
        private static float GOLD_SIZE = 0.25f;         // size of the gold bars

        private const float LABEL_OFFSET_X = 0.08f;     // offset for drawing the number label
        private const float LABEL_OFFSET_Y = 0.38f; 

        private const float GOLD_POSITION_OFFSET = 0.2f; // offset for drawing the gold bar
        
        /// <summary>
        /// Constructor: 
        /// </summary>
        /// <param name="numCells">Total number of game cells (row x column)</param>
        /// <param name="maxColumn">Number of columns</param>
        public GameBoardGUI(int numCells, int maxColumn)
        {
            m_MaxColumn = maxColumn;                        // initialize the variables
            m_TotalNumCells = numCells;
            m_GameBoard = new BoardCellContent[numCells];   // allocate the array
            m_GoldBars = new XNACS1Circle[numCells];

            // Label A: loop through all the game cells
            for (int i = 0; i < m_TotalNumCells; i++)
            {
                m_GameBoard[i] = new BoardCellContent();
                Vector2 pos = GetCellPosition(i);

                // Create the game cell rectangle
                XNACS1Rectangle cell = new XNACS1Rectangle(pos, SIZE, SIZE);
                cell.CenterColor = cell.OutsideColor = Color.LightSeaGreen;
                cell.Label = i.ToString();

                if (SnakeAndLadder.RandomFloat() > 0.3f)
                {  // 30% chance a cell will contain a gold bar
                    m_GameBoard[i] = BoardCellContent.HasNothing;
                    m_GoldBars[i] = null;
                }
                else
                {
                    m_GameBoard[i] = BoardCellContent.HasGold;
                    Vector2 p = new Vector2(pos.X, pos.Y);
                    p.X -= GOLD_POSITION_OFFSET;
                    p.Y -= GOLD_POSITION_OFFSET;
                    m_GoldBars[i] = new XNACS1Circle(p, GOLD_SIZE, "gold");
                }
            }

            // Label B: Initialize the hero
            m_CurrentIndex = 0;                             // initial hero index is 0
            m_Hero = new XNACS1Circle(GetCellPosition(m_CurrentIndex), 0.4f, "kid");
        }

        /// <summary>
        /// Updates current index, wraps around the game baord when index moves
        /// beyond the array size.
        /// </summary>
        /// <param name="num">numnber spaces to move forward by</param>
        public void MoveBy(int num)
        {
            m_CurrentIndex = (m_CurrentIndex + num) % m_TotalNumCells;
            m_Hero.Center = GetCellPosition(m_CurrentIndex);
        }

        /// <summary>
        /// returns the (x,y) position for current index.
        /// </summary>
        /// <param name="position"></param>
        /// <returns></returns>
        private Vector2 GetCellPosition(int index)
        {
            int row = index / m_MaxColumn;               // row is index div column
            int col = index % m_MaxColumn;               // col is index mod column
            Vector2 p = new Vector2();       
            p.Y = row + 0.5f;                               // X is 0.5 offset from row
            p.X = col + 0.5f;                               // Y is 0.5 offset from column
            if (0 != (row % 2))                             // if odd-row
                p.X = m_MaxColumn - p.X;                    // counts backwards
            return p;
        }
    }
}