#region 1. Using Statemetns
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;
#endregion

namespace Othello_Guide_Program
{
    /// <summary>
    ///  This is the main class. Notice this class must subclass from the XNACS1Base
    /// </summary>
    public class Othello_Guide : XNACS1Base
    {
        // Private application state
        // Location and size of the world
        private const float WORLD_MIN_X = -8.0f;          
        private const float WORLD_MIN_Y = -1.0f;        // Lower left corner (-8,-1)
        private const float WORLD_WIDTH = 30.0f;        // width is 30.0

        // Label A: Application state: Circle and red-dot position
        private const int BOARD_RESOLUTION = 6;         // any even number will do
        private XNACS1Circle m_Pointer;                 // red-dot position
        private XNACS1Circle m_WhitePiece;              // white circle position
        private XNACS1Circle m_BlackPiece;              // black circle position
        private PlayBoard m_Board;                      // the 6x6 grid

        // Label B: Support input and output
        private Vector2 m_DeltaMove;                    // accumulate thumbstick input

        private String m_Status;                        // bottom status string
        private String m_TopStatus;
 
#region override base class methods
        /// <summary>
        /// Initialize the application state
        /// </summary>
        protected override void InitializeWorld()
        {
            World.SetWorldCoordinate(new Vector2(WORLD_MIN_X, WORLD_MIN_Y), WORLD_WIDTH);
            World.SetBackgroundColor(Color.LightSkyBlue);

            // Label A: Create the board
            m_Board = new PlayBoard(BOARD_RESOLUTION);      // to draw the grid cells
            
            // Label B: Create the white and black pieces
            m_BlackPiece = new XNACS1Circle(new Vector2(0.5f, 0.5f), 0.5f);
            m_BlackPiece.CenterColor = m_BlackPiece.OutsideColor = Color.Black;
            m_WhitePiece = new XNACS1Circle(new Vector2(1.5f, 1.5f), 0.5f);
            m_WhitePiece.CenterColor = m_WhitePiece.OutsideColor = Color.White;

            // Label C: red current position indicator
            m_Pointer = new XNACS1Circle(new Vector2(0.5f * BOARD_RESOLUTION + 0.5f, 0.5f * BOARD_RESOLUTION + 0.5f), 0.1f);
            m_Pointer.CenterColor = Color.Red;
            m_Pointer.OutsideColor = Color.LightSalmon;

            // Label D: Initial input state
            m_DeltaMove = new Vector2();
            m_DeltaMove.X = 0.0f;     // no initial thumbstick movements
            m_DeltaMove.Y = 0.0f;

            m_TopStatus = "WorldMin=" + World.WorldMin + "  WorldMax=" + World.WorldMax;
        }

        /// <summary>
        /// Update the application state. 
        /// </summary>
        protected override void UpdateWorld()
        {
            // Label A: Compute thumbstick movement
            Vector2 delta = XNACS1Base.GamePad.ThumbSticks.Left;
            m_DeltaMove += (0.4f * delta);  // Accumulates thubmstick movement
            int dx = (int)m_DeltaMove.X;    // Taking the integer component
            int dy = (int)m_DeltaMove.Y;    //   of the thumbstick movement

            if ((dx != 0) || (dy != 0))
            {
                m_DeltaMove.X = 0.0f;       // If movement registered, reset
                m_DeltaMove.Y = 0.0f;       // thumb stick movement
                XNACS1Base.PlayACue( "Move");
            }
            m_Pointer.CenterX += dx;
            m_Pointer.CenterY += dy;

            // Label B: Draws black/white circle with A/B button clicks
            bool aClicked = XNACS1Base.GamePad.ButtonAClicked();
            bool bClicked = XNACS1Base.GamePad.ButtonBClicked();
            if ( aClicked || bClicked ) {
                // Label C: Compute Black/White circle position
                int row = (int)m_Pointer.CenterX;
                int col = (int)m_Pointer.CenterY;
                if ((row >= 0) && (row < BOARD_RESOLUTION) &&  // red-dot in grid bound
                    (col >= 0) && (col < BOARD_RESOLUTION))
                {
                    if (aClicked)                              // Button A clicked
                    {
                        m_WhitePiece.CenterX = row+0.5f;
                        m_WhitePiece.CenterY = col+0.5f;
                        XNACS1Base.PlayACue( "Beep");
                    }
                    else
                    {                                          // Button B clicked
                        m_BlackPiece.CenterX = row+0.5f;
                        m_BlackPiece.CenterY = col+0.5f;
                        XNACS1Base.PlayACue( "Click");
                     }
                     m_Status = "White at:" + m_WhitePiece.Center + "    Black at:" + m_BlackPiece.Center;
                }
                else
                {
                     m_Status = "Position: (" + row + ", " + col + ") is outside the game board. Try again!";
                }
            }
            
            // Allows the default game to exit on Xbox 360 and Windows
            if (XNACS1Base.GamePad.Buttons.Back == ButtonState.Pressed)
                this.Exit();

            EchoToTopStatus(m_TopStatus + @"    Current Position:" + m_Pointer.Center);
            EchoToBottomStatus(m_Status);
        }
#endregion
    }
}
