#region 1. Using Statemetns
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

// Our own library
using XNACS1Lib;
#endregion

namespace BurstABubble_Guide_Program
{
    /// <summary>
    ///  This is the main class. Notice this class must subclass from the XNACS1Base
    /// </summary>
    public class BurstABubble_Guide : XNACS1Base
    {
        // Private application state
        // Location and size of the world
        private const float WORLD_MIN_X = 0.0f;          // lower left corner is (0, 0)
        private const float WORLD_MIN_Y = 0.0f;
        private const float WORLD_WIDTH = 100.0f;        // width of the world is 100.0

        private const float SHOT_RADIUS = 2.0f;             // radius of the lower-left ball
        private const float INIT_SHOT_POS_X = SHOT_RADIUS;  // initial position of the ball
        private const float INIT_SHOT_POS_Y = SHOT_RADIUS;
        private const float DROP_RECT_H = 10.0f;
        private const float DROP_RECT_W = 15.0f;


        private XNACS1Rectangle m_DroppingRect; // The dropping Rectangle
        private XNACS1Circle m_ShotCircle;      // The shooting circle
        private XNACS1Rectangle m_ShootRect;    // The shooting rectangle

        private Vector2 m_ShotV;        // Velocity of the shot
        private Vector2 m_UserV;        // Velocity defined by the thumbStick


        #region override base class methods
        /// <summary>
        /// Initialize the application state
        /// </summary>
        protected override void InitializeWorld()
        {
            World.SetWorldCoordinate(new Vector2(WORLD_MIN_X, WORLD_MIN_Y), WORLD_WIDTH);
            World.SetBackgroundColor(Color.LightPink);

            // A: The dropping rectangle
            Vector2 pos = new Vector2(50.0f, 50.0f);
            m_DroppingRect = new XNACS1Rectangle(pos, DROP_RECT_W, DROP_RECT_H, @"uwb_logo");

            // B: The shot-circle 
            pos.X = INIT_SHOT_POS_X;
            pos.Y = INIT_SHOT_POS_Y;  // Initial position of the shot
            m_ShotCircle = new XNACS1Circle(pos, SHOT_RADIUS);
            m_ShotCircle.CenterColor = Color.White;
            m_ShotCircle.OutsideColor = Color.Black;

            // C: Shot circle and shot-stick velocities
            m_ShotV = new Vector2(0.0f, 0.0f);
            m_UserV = new Vector2(3.0f, 3.0f);

            // D: The shot-stick
            m_ShootRect = new XNACS1Rectangle(new Vector2(0.0f, 0.0f), m_UserV, 1.0f, "");
            m_ShootRect.OutsideColor = m_ShootRect.CenterColor = Color.Green;

        }

        private Vector2 DecreaseVelocity(Vector2 v, float len)
        {
            len *= 0.75f;
            v.Normalize();
            v *= len;
            return v;
        }

        /// <summary>
        /// Update the application state. 
        /// </summary>
        protected override void UpdateWorld()
        {
            float gravity = 0.12f;

            // Label A: if possible, "Sinks" the rectangle
            if (World.CollideWorldBound(m_DroppingRect) == BoundCollideStatus.CollideBottom)
            {
                m_DroppingRect.CenterY = 50.0f;
                XNACS1Base.PlayACue( "BubbleBurst");
            }
            else
                m_DroppingRect.CenterY -= gravity;

            // Label B: collide the circle with the window bounds and decide what to do
            BoundCollideStatus stat = World.ClampAtWorldBound(m_ShotCircle);
            float len = m_ShotV.Length();
            switch (stat)
            {
                    
                case BoundCollideStatus.InsideBound:
                    // Label B1: if completely inside the window bounds
                    if ((len > 0.05f) || (m_ShotCircle.CenterY > (1.5f * INIT_SHOT_POS_Y)))
                    {
                        m_ShotCircle.Center = m_ShotCircle.Center + m_ShotV;
                        m_ShotV.Y -= gravity;
                    }
                    else
                    {
                        // if the circle is moving very slowly, stop it.
                        m_ShotV = new Vector2(0.0f, 0.0f);
                        m_ShotCircle.CenterY = INIT_SHOT_POS_Y;
                    }
                    break;

                case BoundCollideStatus.CollideBottom:
                    // Label B2: bounces off the bottom
                    if ((len < 0.05f) && (m_ShotCircle.CenterY < (1.5f * INIT_SHOT_POS_Y)))
                    {
                        // if the circle is moving very slowly, stop it.
                        m_ShotV = new Vector2(0.0f, 0.0f);
                        m_ShotCircle.CenterY = INIT_SHOT_POS_Y;
                    }
                    else
                    {
                        // bounce the circle
                        m_ShotV = DecreaseVelocity(m_ShotV, len*0.75f);
                        m_ShotV.Y = -m_ShotV.Y;
                    }
                    break;

                case BoundCollideStatus.CollideTop:
                    // Label B3: bounces off the top
                    m_ShotV = DecreaseVelocity(m_ShotV, len);
                    m_ShotV.Y = -m_ShotV.Y;
                    break;

                case BoundCollideStatus.CollideLeft:
                case BoundCollideStatus.CollideRight:
                    // Label B4: bounces off the sides
                    m_ShotV = DecreaseVelocity(m_ShotV, len);
                    m_ShotV.X = -m_ShotV.X;
                    break;
            }


            // Label C: if button-A clicked, re-shoot the circle
            Vector2 aim = XNACS1Base.GamePad.ThumbSticks.Left;
            m_UserV += aim * 0.1f;
            if (XNACS1Base.GamePad.ButtonAClicked())
            {
                // Fire new shot
                m_ShotV = m_UserV;
                m_ShotCircle.CenterX = INIT_SHOT_POS_X;
                m_ShotCircle.CenterY = INIT_SHOT_POS_Y*1.1f;
                XNACS1Base.PlayACue( "Shooting");
            }

            // Label D: manipulating the shooting rectangle
            Vector2 pa = new Vector2(0.0f, 0.0f);
            Vector2 pb = 5.0f * m_UserV;
            m_ShootRect.SetEndPoints(pa, pb, 1.0f);

            EchoToTopStatus(@"Shot Position:" + m_ShotCircle.Center + " ShotV:" + m_ShotV);
            EchoToBottomStatus(@"Rectangle Positions:" + m_DroppingRect.Center);

            // Allows the default game to exit on Xbox 360 and Windows
            if (XNACS1Base.GamePad.Buttons.Back == ButtonState.Pressed)
                this.Exit();

        }
        #endregion
    }
}
