using System;
using System.IO;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace XNACS1Lib
{
    namespace XNACS1Lib.GamePadSupport
    {
        /// <summary>
        /// Records the states (pressed or release) of all the buttons on a XBOX 360 game pad.
        /// </summary>
        /// 
        public struct AllButtonsOnGamePad
        {
            /// <summary>
            /// State of the A-Button (pressed or released).
            /// </summary>
            public ButtonState A;

            /// <summary>
            /// State of the B-Button (pressed or released).
            /// </summary>
            public ButtonState B;

            /// <summary>
            /// State of the back-Button (pressed or released).
            /// </summary>
            public ButtonState Back;

            /// <summary>
            /// State of the leftShoulder-Button (pressed or released).
            /// </summary>
            public ButtonState LeftShoulder;

            /// <summary>
            /// State of the LeftStick-Button (pressed or released).
            /// </summary>
            public ButtonState LeftStick;

            /// <summary>
            /// State of the RightShoulder-Button (pressed or released).
            /// </summary>
            public ButtonState RightShoulder;

            /// <summary>
            /// State of the RightStick-Button (pressed or released).
            /// </summary>
            public ButtonState RightStick;

            /// <summary>
            /// State of the Start-Button (pressed or released).
            /// </summary>
            public ButtonState Start;

            /// <summary>
            /// State of the X-Button (pressed or released).
            /// </summary>
            public ButtonState X;

            /// <summary>
            /// State of the Y-Button (pressed or released).
            /// </summary>
            public ButtonState Y;
        }

        /// <summary>
        /// States (depressed or released) of the DPad-buttons (left, right, top, down)
        /// on the XBOX 360 game pad.
        /// </summary>
        public struct DPadOnGamePad
        {
            /// <summary>
            /// State of the DPad Down-Button (pressed or released).
            /// </summary>
            public ButtonState Down;

            /// <summary>
            /// State of the DPad Left-Button (pressed or released).
            /// </summary>
            public ButtonState Left;

            /// <summary>
            /// State of the DPad Right-Button (pressed or released).
            /// </summary>
            public ButtonState Right;


            /// <summary>
            /// State of the DPad Up-Button (pressed or released).
            /// </summary>
            public ButtonState Up;
        }

        /// <summary>
        /// Positions of the left and right thumbSticks on the XBOX 360 controller.
        /// </summary>
        public struct ThumbSticksOnGamePad
        {
            /// <summary>
            /// Position of the left thumbStick (X/Y values between -1 and +1).
            /// </summary>
            public Vector2 Left;


            /// <summary>
            /// Position of the right thumbStick (X/Y values between -1 and +1).
            /// </summary>
            public Vector2 Right;

        }

        /// <summary>
        /// Positions of the left and right triggers on the XBOX 360 controller.
        /// </summary>
        public struct TriggersOnGamePad
        {
            /// <summary>
            /// Position of the left trigger (value between 0 to 1).
            /// </summary>
            public float Left;

            /// <summary>
            /// Position of the right trigger (value betwen 0 to 1).
            /// </summary>
            public float Right;
        }

        /// <summary>
        /// 
        /// </summary>
        internal class ButtonClickOnGamePad
        {
            public bool m_ButtonPressed;
            public bool m_EventValid;

            internal ButtonClickOnGamePad()
            {
                m_ButtonPressed = false;
                m_EventValid = false;
            }
            internal void SetButtonEvent(ButtonState aButton)
            {
                m_EventValid = ((aButton == ButtonState.Pressed) && (!m_ButtonPressed));
                m_ButtonPressed = (aButton == ButtonState.Pressed);

            }
            internal bool ButtonHasClicked()
            {
                bool canUseEvent = m_EventValid;
                m_EventValid = false;
                return canUseEvent && m_ButtonPressed;
            }
        }
 

    /// <summary>
    /// State information for all input mechanisms on the XBOX 360 game pad controller.
    /// </summary>
    public class GamePadStruct
    {
        internal GamePadStruct(float v)
        {
            m_VibrationA = v;
            m_VibrationB = v;

            Buttons = new XNACS1Lib.GamePadSupport.AllButtonsOnGamePad();
            Dpad = new XNACS1Lib.GamePadSupport.DPadOnGamePad();
            ThumbSticks = new XNACS1Lib.GamePadSupport.ThumbSticksOnGamePad();
            Triggers = new XNACS1Lib.GamePadSupport.TriggersOnGamePad();
            aButton = new XNACS1Lib.GamePadSupport.ButtonClickOnGamePad();
            bButton = new XNACS1Lib.GamePadSupport.ButtonClickOnGamePad();
            xButton = new XNACS1Lib.GamePadSupport.ButtonClickOnGamePad();
            yButton = new XNACS1Lib.GamePadSupport.ButtonClickOnGamePad();
            startButton = new XNACS1Lib.GamePadSupport.ButtonClickOnGamePad();
            backButton = new XNACS1Lib.GamePadSupport.ButtonClickOnGamePad();
        }

        /// <summary>
        /// States of all the buttons on the XBOX 360 game pad.
        /// </summary>
        public XNACS1Lib.GamePadSupport.AllButtonsOnGamePad Buttons;

        /// <summary>
        /// States of all the buttons on the DPad on the XBOX 360 game pad.
        /// </summary>
        public XNACS1Lib.GamePadSupport.DPadOnGamePad Dpad;

        /// <summary>
        /// Positions of the left and right thumbsticks on the XBOX 360 game pad.
        /// </summary>
        public XNACS1Lib.GamePadSupport.ThumbSticksOnGamePad ThumbSticks;

        /// <summary>
        /// Positions of the left and right triggers on the XBOX 360 game pad.
        /// </summary>
        public XNACS1Lib.GamePadSupport.TriggersOnGamePad Triggers;

        internal float m_VibrationA, m_VibrationB;

        internal XNACS1Lib.GamePadSupport.ButtonClickOnGamePad aButton, bButton, xButton, yButton, startButton, backButton;

        internal void UpdateButtonClick()
        {
            aButton.SetButtonEvent(Buttons.A);
            bButton.SetButtonEvent(Buttons.B);
            xButton.SetButtonEvent(Buttons.X);
            yButton.SetButtonEvent(Buttons.Y);
            startButton.SetButtonEvent(Buttons.Start);
            backButton.SetButtonEvent(Buttons.Back);
        }

        /// <summary>
        /// If Button-A has been clicked.
        /// </summary>
        /// <returns>T/F on if the button has been clicked.</returns>
        public bool ButtonAClicked() { return aButton.ButtonHasClicked(); }

        /// <summary>
        /// If Button-B has been clicked.
        /// </summary>
        /// <returns>T/F on if the button has been clicked.</returns>
        public bool ButtonBClicked() { return bButton.ButtonHasClicked(); }

        /// <summary>
        /// If Button-X has been clicked.
        /// </summary>
        /// <returns>T/F on if the button has been clicked.</returns>
        public bool ButtonXClicked() { return xButton.ButtonHasClicked(); }

        /// <summary>
        /// If Button-Y has been clicked.
        /// </summary>
        /// <returns>T/F on if the button has been clicked.</returns>
        public bool ButtonYClicked() { return yButton.ButtonHasClicked(); }

        /// <summary>
        /// If the start button has been clicked.
        /// </summary>
        /// <returns>T/F on if the button has been clicked.</returns>
        public bool ButtonStartClicked() { return startButton.ButtonHasClicked(); }

        /// <summary>
        /// If the back button has been clicked.
        /// </summary>
        /// <returns>T/F on if the button has been clicked.</returns>
        public bool ButtonBackClicked() { return backButton.ButtonHasClicked(); }
    }
}
    internal static class XNACS1LibInputManager
    {

        /// <summary>
        /// Set the vibration level on the game pad. With keyboard only, this function does not
        /// do anything.
        /// </summary>
        /// <param name="index">game pad index (1 to 4)</param>
        /// <param name="a">motor-a: value between 0.0f to 1.0f</param>
        /// <param name="b">motor-b: value between 0.0f to 1.0f</param>
        /// <returns></returns>
        public static void SetGamePadVibration(PlayerIndex index, float a, float b)
        {
            // for now, gamepad index-i is ignored.
            //
            m_MyGamePad.m_VibrationA = a;
            m_MyGamePad.m_VibrationB = b;
        }

        public static void UpdateGamePadState()
        {
            // Only get game controller input if there actually is one :)
            if (Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).IsConnected)
            {
                // Buttons
                m_MyGamePad.Buttons.A = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).Buttons.A;
                m_MyGamePad.Buttons.B = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).Buttons.B;
                m_MyGamePad.Buttons.X = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).Buttons.X;
                m_MyGamePad.Buttons.Y = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).Buttons.Y;

                m_MyGamePad.Buttons.Back = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).Buttons.Back;
                m_MyGamePad.Buttons.Start = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).Buttons.Start;

                m_MyGamePad.Buttons.LeftShoulder = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).Buttons.LeftShoulder;
                m_MyGamePad.Buttons.RightShoulder = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).Buttons.RightShoulder;

                m_MyGamePad.Buttons.LeftStick = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).Buttons.LeftStick;
                m_MyGamePad.Buttons.RightStick = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).Buttons.RightStick;

                // DPad
                m_MyGamePad.Dpad.Up = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).DPad.Up;
                m_MyGamePad.Dpad.Down = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).DPad.Down;
                m_MyGamePad.Dpad.Left = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).DPad.Left;
                m_MyGamePad.Dpad.Right = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).DPad.Right;

                // ThumbSticks
                m_MyGamePad.ThumbSticks.Left = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).ThumbSticks.Left;
                m_MyGamePad.ThumbSticks.Right = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).ThumbSticks.Right;

                // Triggers
                m_MyGamePad.Triggers.Left = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).Triggers.Left;
                m_MyGamePad.Triggers.Right = Microsoft.Xna.Framework.Input.GamePad.GetState(PlayerIndex.One).Triggers.Right;

                // Vibration
                Microsoft.Xna.Framework.Input.GamePad.SetVibration(PlayerIndex.One, m_MyGamePad.m_VibrationA, m_MyGamePad.m_VibrationB);
            }
            else // reset it, for the keyboard, below
            {
                m_MyGamePad.Buttons.A = ButtonState.Released;
                m_MyGamePad.Buttons.B = ButtonState.Released;
                m_MyGamePad.Buttons.X = ButtonState.Released;
                m_MyGamePad.Buttons.Y = ButtonState.Released;

                m_MyGamePad.Buttons.Back = ButtonState.Released;
                m_MyGamePad.Buttons.Start = ButtonState.Released;

                m_MyGamePad.Buttons.LeftShoulder = ButtonState.Released;
                m_MyGamePad.Buttons.RightShoulder = ButtonState.Released;

                m_MyGamePad.Buttons.LeftStick = ButtonState.Released;
                m_MyGamePad.Buttons.RightStick = ButtonState.Released;

                // DPad
                m_MyGamePad.Dpad.Up = ButtonState.Released;
                m_MyGamePad.Dpad.Down = ButtonState.Released;
                m_MyGamePad.Dpad.Left = ButtonState.Released;
                m_MyGamePad.Dpad.Right = ButtonState.Released;

                // ThumbSticks
                m_MyGamePad.ThumbSticks.Left = new Vector2(0, 0);
                m_MyGamePad.ThumbSticks.Right = new Vector2(0, 0);

                // Triggers
                m_MyGamePad.Triggers.Left = 0f;
                m_MyGamePad.Triggers.Right = 0f;
            }

            //  update from the keyboard, so that if either kdb or controller is being pressed, 
            //      the game gets the 'pressed' message.  We only grab keyboard input if the
            //      controller hasn't done anything yet, so that the controller has 'precedence'
            if (m_MyGamePad.Buttons.A == ButtonState.Released)
                m_MyGamePad.Buttons.A = Keyboard.GetState().IsKeyDown(Keys.A) ? ButtonState.Pressed : ButtonState.Released;
            if (m_MyGamePad.Buttons.B == ButtonState.Released)
                m_MyGamePad.Buttons.B = Keyboard.GetState().IsKeyDown(Keys.B) ? ButtonState.Pressed : ButtonState.Released;
            if (m_MyGamePad.Buttons.X == ButtonState.Released)
                m_MyGamePad.Buttons.X = Keyboard.GetState().IsKeyDown(Keys.X) ? ButtonState.Pressed : ButtonState.Released;
            if (m_MyGamePad.Buttons.Y == ButtonState.Released)
                m_MyGamePad.Buttons.Y = Keyboard.GetState().IsKeyDown(Keys.Y) ? ButtonState.Pressed : ButtonState.Released;

            if (m_MyGamePad.Buttons.Back == ButtonState.Released)
                m_MyGamePad.Buttons.Back = Keyboard.GetState().IsKeyDown(Keys.Q) ? ButtonState.Pressed : ButtonState.Released;
            if (m_MyGamePad.Buttons.Start == ButtonState.Released)
                m_MyGamePad.Buttons.Start = Keyboard.GetState().IsKeyDown(Keys.W) ? ButtonState.Pressed : ButtonState.Released;

            if (m_MyGamePad.Buttons.LeftShoulder == ButtonState.Released)
                m_MyGamePad.Buttons.LeftShoulder = Keyboard.GetState().IsKeyDown(Keys.Y) ? ButtonState.Pressed : ButtonState.Released;
            if (m_MyGamePad.Buttons.RightShoulder == ButtonState.Released)
                m_MyGamePad.Buttons.RightShoulder = Keyboard.GetState().IsKeyDown(Keys.U) ? ButtonState.Pressed : ButtonState.Released;

            if (m_MyGamePad.Buttons.LeftStick == ButtonState.Released)
                m_MyGamePad.Buttons.LeftStick = Keyboard.GetState().IsKeyDown(Keys.N) ? ButtonState.Pressed : ButtonState.Released;
            if (m_MyGamePad.Buttons.RightStick == ButtonState.Released)
                m_MyGamePad.Buttons.RightStick = Keyboard.GetState().IsKeyDown(Keys.L) ? ButtonState.Pressed : ButtonState.Released;

            // DPad
            if (m_MyGamePad.Dpad.Up == ButtonState.Released)
                m_MyGamePad.Dpad.Up = Keyboard.GetState().IsKeyDown(Keys.R) ? ButtonState.Pressed : ButtonState.Released;
            if (m_MyGamePad.Dpad.Down == ButtonState.Released)
                m_MyGamePad.Dpad.Down = Keyboard.GetState().IsKeyDown(Keys.C) ? ButtonState.Pressed : ButtonState.Released;

            if (m_MyGamePad.Dpad.Left == ButtonState.Released)
                m_MyGamePad.Dpad.Left = Keyboard.GetState().IsKeyDown(Keys.D) ? ButtonState.Pressed : ButtonState.Released;
            if (m_MyGamePad.Dpad.Right == ButtonState.Released)
                m_MyGamePad.Dpad.Right = Keyboard.GetState().IsKeyDown(Keys.F) ? ButtonState.Pressed : ButtonState.Released;

            // ThumbSticks
            if (m_MyGamePad.ThumbSticks.Left.X == 0 && m_MyGamePad.ThumbSticks.Left.Y == 0)
            {
                Vector2 left = new Vector2();
                left.X = Keyboard.GetState().IsKeyDown(Keys.K) ? 0.75f : 0.0f;
                left.X -= Keyboard.GetState().IsKeyDown(Keys.J) ? 0.75f : 0.0f;
                left.Y = Keyboard.GetState().IsKeyDown(Keys.I) ? 0.75f : 0.0f;
                left.Y -= Keyboard.GetState().IsKeyDown(Keys.M) ? 0.75f : 0.0f;
                m_MyGamePad.ThumbSticks.Left = left;
            }

            if (m_MyGamePad.ThumbSticks.Right.X == 0 && m_MyGamePad.ThumbSticks.Right.Y == 0)
            {
                Vector2 right = new Vector2();
                right.X = Keyboard.GetState().IsKeyDown(Keys.Right) ? 0.75f : 0.0f;
                right.X -= Keyboard.GetState().IsKeyDown(Keys.Left) ? 0.75f : 0.0f;
                right.Y = Keyboard.GetState().IsKeyDown(Keys.Up) ? 0.75f : 0.0f;
                right.Y -= Keyboard.GetState().IsKeyDown(Keys.Down) ? 0.75f : 0.0f;
                m_MyGamePad.ThumbSticks.Right = right;
            }

            // Triggers
            if (m_MyGamePad.Triggers.Left == 0)
                m_MyGamePad.Triggers.Left = Keyboard.GetState().IsKeyDown(Keys.RightAlt) ? 0.5f : 0.0f;
            if (m_MyGamePad.Triggers.Right == 0)
                m_MyGamePad.Triggers.Right = Keyboard.GetState().IsKeyDown(Keys.RightControl) ? 0.5f : 0.0f;

            m_MyGamePad.UpdateButtonClick();
        }

        private static XNACS1Lib.GamePadSupport.GamePadStruct m_MyGamePad = new XNACS1Lib.GamePadSupport.GamePadStruct(0.0f);
        public static XNACS1Lib.GamePadSupport.GamePadStruct GamePad { get { return m_MyGamePad; } }
    }
}