using System;
using System.IO;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace XNACS1Lib
{
    /// <summary>
    /// This is the base class for programming assignments.
    /// </summary>
    internal class XNACS1LibDrawHelper
    {
        #region static support for drawing the circle
        private const int NUM_CIRCLE_POINTS = 37;
        private static VertexPositionColorTexture[] mCircleVertices = null;
        private static void InitializeCircle()
        {
            if (null != mCircleVertices)
                return;

            float theta = (2.0f * (float)Math.PI) / NUM_CIRCLE_POINTS;
            mCircleVertices = new VertexPositionColorTexture[NUM_CIRCLE_POINTS + 2];
            mCircleVertices[0].Position = new Vector3(0.0f, 0.0f, 0.0f);
            mCircleVertices[0].Color = Color.Cyan;
            mCircleVertices[0].TextureCoordinate = new Vector2(0.5f, 0.5f);

            for (int i = 0; i < NUM_CIRCLE_POINTS; i++)
            {
                float s, c;
                s = (float)Math.Sin(-i * theta);
                c = (float)Math.Cos(-i * theta);
                mCircleVertices[i + 1].Position = new Vector3(s, c, 0.0f);
                mCircleVertices[i + 1].Color = Color.Magenta;
                mCircleVertices[i + 1].TextureCoordinate = new Vector2(((s + 1) * 0.5f), (1 - ((c + 1) * 0.5f)));
            }
            mCircleVertices[NUM_CIRCLE_POINTS + 1] = mCircleVertices[1];
        }
        #endregion

        #region static support for drawing rectangle
        private const int NUM_RECTANGLE_POINTS = 4;
        private static VertexPositionColorTexture[] mRecVertex;
        private static void InitializeRectangle()
        {
            if (null != mRecVertex)
                return;

            mRecVertex = new VertexPositionColorTexture[NUM_RECTANGLE_POINTS + 2];
            mRecVertex[0].Position = new Vector3(0.5f, 0.0f, 0.0f);
            mRecVertex[0].TextureCoordinate = new Vector2(0.5f, 0.5f);

            mRecVertex[1].Position = new Vector3(0.0f, 0.5f, 0.0f);
            mRecVertex[1].TextureCoordinate = new Vector2(0.0f, 0.0f);

            mRecVertex[2].Position = new Vector3(0.0f, -0.5f, 0.0f);
            mRecVertex[2].TextureCoordinate = new Vector2(0.0f, 1.0f);

            mRecVertex[3].Position = new Vector3(1.0f, -0.5f, 0.0f);
            mRecVertex[3].TextureCoordinate = new Vector2(1.0f, 1.0f);

            mRecVertex[4].Position = new Vector3(1.0f, 0.5f, 0.0f);
            mRecVertex[4].TextureCoordinate = new Vector2(1.0f, 0.0f);

            for (int i = 0; i <= NUM_RECTANGLE_POINTS; i++)
                mRecVertex[i].Color = Color.DarkGoldenrod;

            mRecVertex[5] = mRecVertex[1];
        }
        #endregion support for rectangle

        private TextureTable m_TexTable;
        private XNACS1LibFontManager m_FontManager;
        private Vector2 m_WorldMin, m_WorldMax;
        private Vector2 m_DeviceSize;

        private GraphicsDevice m_GraphicsDevice;
        private BasicEffect m_BasicEffect;
        private EffectPool m_EffectPool;

        private List<XNACS1Primitive> m_DrawSet =null;

        public XNACS1LibDrawHelper(GraphicsDevice graphicsDevice, ContentManager c, XNACS1Rectangle worldBound, XNACS1LibFontManager fontManager, Vector2 deviceSize)
        {
            m_DrawSet = new List<XNACS1Primitive>();
            
            m_GraphicsDevice = graphicsDevice;
            m_GraphicsDevice.RenderState.DepthBufferEnable = false;
            m_GraphicsDevice.RenderState.MultiSampleAntiAlias = true;
            m_GraphicsDevice.RenderState.CullMode = CullMode.None;
            m_GraphicsDevice.RenderState.FillMode = FillMode.Solid;
            m_GraphicsDevice.RenderState.AlphaBlendEnable = true;
            m_GraphicsDevice.RenderState.SourceBlend = Blend.SourceAlpha;
            m_GraphicsDevice.RenderState.DestinationBlend = Blend.InverseSourceAlpha;
            m_GraphicsDevice.RenderState.BlendFunction = BlendFunction.Add;
            m_GraphicsDevice.VertexDeclaration = new VertexDeclaration(m_GraphicsDevice, VertexPositionColorTexture.VertexElements);

            m_EffectPool = new EffectPool();
            m_BasicEffect = new BasicEffect(m_GraphicsDevice, m_EffectPool);
            m_BasicEffect.VertexColorEnabled = true;
            m_BasicEffect.LightingEnabled = false;

            //
            SetTransformationMatrices(worldBound, deviceSize);
            
            m_TexTable = new TextureTable(c);
            m_FontManager = fontManager;

            InitializeCircle();
            InitializeRectangle();
        }

        public void SetTransformationMatrices(XNACS1Rectangle worldBound, Vector2 deviceSize)
        {

            Vector2 wMin = worldBound.MinBound;
            Vector2 wMax = worldBound.MaxBound;

            Vector2 size = wMax - wMin;
            Matrix w2n, toOrg, toNDC, reCenter;
            toOrg = Matrix.CreateTranslation(-wMin.X, -wMin.Y, 0.0f);
            toNDC = Matrix.CreateScale(2.0f / size.X, 2.0f / size.Y, 1.0f);
            reCenter = Matrix.CreateTranslation(-1.0f, -1.0f, 0.0f);
            w2n = toOrg * toNDC * reCenter;
            m_BasicEffect.View = w2n;
            m_BasicEffect.World = Matrix.Identity;
            m_BasicEffect.Projection = Matrix.Identity;
            m_BasicEffect.CommitChanges();
            m_WorldMin = wMin;
            m_WorldMax = wMax;
            m_DeviceSize = deviceSize;
        }

        public void EnableTexture(String name)
        {
            if (null == name)
            {
                DisableTexture();
            }
            else
            {
                Texture2D tex = m_TexTable.FindTexture(name);
                if (tex == null)
                {
                    m_TexTable.LoadTexture(name);
                    tex = m_TexTable.FindTexture(name);
                }

                if (tex != null)
                {
                    m_BasicEffect.Texture = tex;
                    m_BasicEffect.TextureEnabled = true;
                    m_BasicEffect.VertexColorEnabled = false;
                }
                else
                {
                    DisableTexture();
                }
            }
            m_BasicEffect.CommitChanges();
        }

        public void DisableTexture()
        {
            m_BasicEffect.Texture = null;
            m_BasicEffect.TextureEnabled = false;
            m_BasicEffect.VertexColorEnabled = true;
            m_BasicEffect.CommitChanges();
        }

        public void SetWorldXform(Vector2 scale, float rotateInDegree, Vector2 translate, Vector2 pivot)
        {
            Matrix pm, tm, sm, rm;
            pm = Matrix.CreateTranslation(-pivot.X, -pivot.Y, 0.0f);
            tm = Matrix.CreateTranslation(translate.X + pivot.X, translate.Y + pivot.Y, 0.0f);
            rm = Matrix.CreateRotationZ((float)(rotateInDegree * (Math.PI / 180.0f)));
            sm = Matrix.CreateScale(scale.X, scale.Y, 1.0f);
            m_BasicEffect.World = pm * sm * rm * tm;
            m_BasicEffect.CommitChanges();
        }
        public void RestorWorldXform()
        {
            m_BasicEffect.World = Matrix.Identity;
            m_BasicEffect.CommitChanges();
        }

        public void SetXform(Vector2 scale, float rotateInDegree, Vector2 translate)
        {
            SetWorldXform(scale, rotateInDegree, translate, new Vector2(0.0f, 0.0f));
        }

        /// <summary>
        /// Draws a circle.
        /// </summary>
        /// <param name="center">Center of the circle.</param>
        /// <param name="radius">Radius of the circle.</param>
        /// <param name="rotation">Rotation of the circle (about the center, in degree).</param>
        /// <param name="outsideColor">Color on the circuference.</param>
        /// <param name="centerColor">Color at the center of the circle.</param>
        /// <param name="texName">Name of texture (use null for none) for the circle.</param>
        public void DrawCircle(Vector2 center, float radius, float rotation, Color outsideColor, Color centerColor, String texName)
        {
            for (int i = 1; i <= NUM_CIRCLE_POINTS + 1; i++)
                mCircleVertices[i].Color = outsideColor;
            mCircleVertices[0].Color = centerColor;

            SetXform(new Vector2(radius, radius), rotation, center);
            EnableTexture(texName);
            m_GraphicsDevice.DrawUserPrimitives(PrimitiveType.TriangleFan, mCircleVertices, 0, NUM_CIRCLE_POINTS);
        }

        /// <summary>
        /// Draws a general non-axis aligned rectangle.
        /// </summary>
        /// <param name="pa">One end of rectangle (center of edge 1).</param>
        /// <param name="pb">Opposite end of rectangle (center of edge 2).</param>
        /// <param name="edgeWidth">Width of the edges where pa and pb are located (length of the rectangle is defined by the distance between pa and pb).</param>
        /// <param name="outsideColor">Color on the circuference.</param>
        /// <param name="centerColor">Color at the center of the circle.</param>
        /// <param name="texName">Name of texture (use null for none) for the rectangle.</param>
        public void DrawRectangle(Vector2 pa, Vector2 pb, float edgeWidth, Color outsideColor, Color centerColor, String texName)
        {

            Vector2 delta = pb - pa;
            float length = delta.Length();
            if (length > 0.001f)
            {
                // DrawCircle(pa, edgeWidth, 0.0f, Color.Black, Color.Black, null);
                // DrawCircle(pb, edgeWidth, 0.0f, Color.Green, Color.Green, null);

                delta /= length;
                float rot = (float)Math.Acos((double)delta.X);
                rot *= 180.0f / (float)Math.PI;
                if (delta.X > 0.0f)
                {
                    if (delta.Y < 0.0f)
                        rot = -rot;
                }
                else
                {
                    if (delta.Y < 0.0f)
                        rot = -rot;
                }

                Vector2 size = new Vector2(length, edgeWidth);

                for (int i = 1; i <= NUM_RECTANGLE_POINTS + 1; i++)
                    mRecVertex[i].Color = outsideColor;
                mRecVertex[0].Color = centerColor;

                SetXform(size, rot, pa);
                EnableTexture(texName);
                m_GraphicsDevice.DrawUserPrimitives(PrimitiveType.TriangleFan, mRecVertex, 0, NUM_RECTANGLE_POINTS);
            }

        }

        /// <summary>
        /// Draws a roated rectangle.
        /// </summary>
        /// <param name="center">Center of the rectangle.</param>
        /// <param name="xWidth">Horizontal width of the rectangle.</param>
        /// <param name="yHeight">Vertical height of the rectangle.</param>
        /// <param name="rot">Rotation in degrees.</param>
        /// <param name="outsideColor">Color on the circuference.</param>
        /// <param name="centerColor">Color at the center of the circle.</param>
        /// <param name="texName">Name of texture (use null for none) for the rectangle.</param>
        public void DrawRectangle(Vector2 center, float xWidth, float yHeight, float rot, Color outsideColor, Color centerColor, String texName)
        {
            Vector2 size = new Vector2(xWidth, yHeight);
            for (int i = 1; i <= NUM_RECTANGLE_POINTS + 1; i++)
                mRecVertex[i].Color = outsideColor;
            mRecVertex[0].Color = centerColor;

            center.X -= 0.5f;
            SetWorldXform(new Vector2(xWidth, yHeight), rot, center, new Vector2(0.5f, 0.0f));
            EnableTexture(texName);
            m_GraphicsDevice.DrawUserPrimitives(PrimitiveType.TriangleFan, mRecVertex, 0, NUM_RECTANGLE_POINTS);
        }

        public void DrawLineSegments(Vector2 a, Vector2 b)
        {
            VertexPositionColorTexture[] v = new VertexPositionColorTexture[2];
            v[0] = new VertexPositionColorTexture(
                        new Vector3(a.X, a.Y, 0.0f), Color.White, new Vector2(0.0f, 0.0f));
            v[1] = new VertexPositionColorTexture(
                        new Vector3(b.X, b.Y, 0.0f), Color.White, new Vector2(1.0f, 1.0f));
            SetWorldXform(new Vector2(1.0f, 1.0f), 0.0f, new Vector2(0.0f, 0.0f), new Vector2(0.0f, 0.0f));
            m_GraphicsDevice.DrawUserPrimitives(PrimitiveType.LineList, v, 0, 1);
        }

        public void BeginDraw()
        {
            m_BasicEffect.Begin();
            m_BasicEffect.Techniques[0].Passes[0].Begin();
        }

        public void EndDraw()
        {
            m_BasicEffect.Techniques[0].Passes[0].End();
            m_BasicEffect.End();
        }

        private Vector2 WCToDC(Vector2 wc)
        {
            Vector2 dc = new Vector2();
            Vector2 wcSize = m_WorldMax - m_WorldMin;
            dc.X = (wc.X - m_WorldMin.X) * m_DeviceSize.X / wcSize.X;
            dc.Y = ((wc.Y - m_WorldMin.Y) * m_DeviceSize.Y / wcSize.Y);
            dc.Y = m_DeviceSize.Y - dc.Y;
            return dc;
        }

        public void DrawFontAt(Vector2 wcAt, String msg, Color c)
        {
            m_FontManager.DrawFontsAt(WCToDC(wcAt), msg, c);
        }

        #region draw set support
        public void AddToSet(XNACS1Primitive p)
        {
            if (null != m_DrawSet)
            {
                if (!m_DrawSet.Contains(p))
                    m_DrawSet.Add(p);
            }
        }

        public void RemoveFromSet(XNACS1Primitive p)
        {
            if (null != m_DrawSet)
                m_DrawSet.Remove(p);
        }

        public void RemoveAllFromSet()
        {
            if (null != m_DrawSet)
                m_DrawSet.Clear();
        }

        public void TopOfDrawSet(XNACS1Primitive p)
        {
            if (null != m_DrawSet)
            {
                if (IsInDrawSet(p))
                {
                    m_DrawSet.Remove(p);
                    m_DrawSet.Add(p);
                }
            }
        }

        public void DrawAll()
        {
            if (null != m_DrawSet)
            {
                foreach (XNACS1Primitive p in m_DrawSet)
                    p.Draw();
            }
        }

        public void TravelAll()
        {
            foreach (XNACS1Primitive p in m_DrawSet)
                p.TravelPrimitive();

        }

        public bool IsInDrawSet(XNACS1Primitive p)
        {
            return m_DrawSet.Contains(p);
        }

        public int DrawSetSize()
        {
            return m_DrawSet.Count;
        }
        #endregion draw set support
        
    }
}